package com.tidecloud.dataacceptance.service;

import javax.xml.bind.DatatypeConverter;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import io.netty.buffer.ByteBuf;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;

public abstract class HexBinaryAcceptanceHandlerAdapter extends AcceptanceInboundHandlerAdapter {
	private static final Logger logger = LoggerFactory.getLogger(HexBinaryAcceptanceHandlerAdapter.class);
	@Override
	public void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
		super.channelRead(ctx, msg);
		ByteBuf in = (ByteBuf) msg;
		if (!in.isReadable()) {
			return;
		}
		printAcceptanceData(in,ctx);
		
		try {
			handle(in,ctx.channel());
		} catch (Exception e) {
			logger.error(e.getMessage());
		} finally {
			in.release();
			
		}
		
		ctx.flush();
	}

	private void printAcceptanceData(ByteBuf dataByteBuf,ChannelHandlerContext ctx) {
		ByteBuf dataByteBufCopy = dataByteBuf.copy();
		byte[] dataByteArray = new byte[dataByteBufCopy.readableBytes()];
		dataByteBufCopy.readBytes(dataByteArray);
		String printHexBinary = DatatypeConverter.printHexBinary(dataByteArray);
		logger.info("设备: [{}] 传入数据为 : {}", channelDeviceMap.get(ctx.channel()), printHexBinary);
		dataByteBufCopy.release();
	}

	abstract protected void handle(ByteBuf in, Channel channel) throws Exception;
	
}
